'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

exports.default = format;
exports.format_national_number = format_national_number;
exports.local_to_international_style = local_to_international_style;

var _common = require('./common');

var _metadata = require('../metadata.min');

var _metadata2 = _interopRequireDefault(_metadata);

var _metadata3 = require('./metadata');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function format(number, format, third_argument) {
	// If the first argument object is expanded
	if (typeof number === 'string') {
		number = { phone: number, country: format };
		format = third_argument;
	}

	var country_metadata = _metadata2.default.countries[number.country];

	switch (format) {
		case 'International':
			var national_number = format_national_number(number.phone, 'International', country_metadata);
			return '+' + (0, _metadata3.get_phone_code)(country_metadata) + ' ' + national_number;

		case 'International_plaintext':
			return '+' + (0, _metadata3.get_phone_code)(country_metadata) + number.phone;

		case 'National':
			return format_national_number(number.phone, 'National', country_metadata);
	}
}

// This was originally set to $1 but there are some countries for which the
// first group is not used in the national pattern (e.g. Argentina) so the $1
// group does not match correctly.  Therefore, we use \d, so that the first
// group actually used in the pattern will be matched.
// This is a port of Google Android `libphonenumber`'s
// `phonenumberutil.js` of 17th November, 2016.
//
// https://github.com/googlei18n/libphonenumber/commits/master/javascript/i18n/phonenumbers/phonenumberutil.js

var FIRST_GROUP_PATTERN = /(\$\d)/;

function format_national_number(number, format_as, country_metadata) {
	var format = choose_format_for_number((0, _metadata3.get_formats)(country_metadata), number);

	if (!format) {
		return number;
	}

	var formatting_rule = (0, _metadata3.get_format_international_format)(format);
	var pattern_to_match = new RegExp((0, _metadata3.get_format_pattern)(format));

	var national_prefix_formatting_rule = (0, _metadata3.get_format_national_prefix_formatting_rule)(format, country_metadata);

	if (format_as === 'National' && !(0, _metadata3.get_format_national_prefix_is_optional_when_formatting)(format, country_metadata) && national_prefix_formatting_rule) {
		return number.replace(pattern_to_match, formatting_rule.replace(FIRST_GROUP_PATTERN, national_prefix_formatting_rule));
	}

	var formatted_number = number.replace(pattern_to_match, formatting_rule);

	if (format_as === 'International') {
		return local_to_international_style(formatted_number);
	}

	return formatted_number;
}

function choose_format_for_number(available_formats, national_number) {
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = (0, _getIterator3.default)(available_formats), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var _format = _step.value;

			if ((0, _metadata3.get_format_leading_digits)(_format)) {
				// The last leading_digits_pattern is used here, as it is the most detailed
				var last_leading_digits_pattern = (0, _metadata3.get_format_leading_digits)(_format)[(0, _metadata3.get_format_leading_digits)(_format).length - 1];

				if (national_number.search(last_leading_digits_pattern) !== 0) {
					return;
				}
			}

			if ((0, _common.matches_entirely)(new RegExp((0, _metadata3.get_format_pattern)(_format)), national_number)) {
				return _format;
			}
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}
}

// Removes brackets and replaces dashes with spaces.
//
// E.g. "(999) 111-22-33" -> "999 111 22 33"
//
function local_to_international_style(local) {
	return local
	// Remove brackets
	.replace(/[\(\)]/g, '')
	// Replace dashes with spaces
	.replace(/\-/g, ' ').trim();
}
//# sourceMappingURL=format.js.map