'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.default = parse;
exports.normalize = normalize;
exports.replace_characters = replace_characters;
exports.extract_possible_number = extract_possible_number;
exports.extract_country_phone_code = extract_country_phone_code;
exports.get_metadata_by_country_phone_code = get_metadata_by_country_phone_code;
exports.strip_national_prefix = strip_national_prefix;
exports.find_country_code = find_country_code;
exports.is_national_phone_number = is_national_phone_number;

var _common = require('./common');

var _metadata = require('../metadata.min');

var _metadata2 = _interopRequireDefault(_metadata);

var _metadata3 = require('./metadata');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var default_options = {
	country: {}
};

// `options`:
//  {
//    country:
//    {
//      restrict - (a two-letter country code)
//                 the phone number must be in this country
//
//      default - (a two-letter country code)
//                default country to use for phone number parsing and validation
//                (if no country code could be derived from the phone number)
//    }
//  }
//
// Returns `{ country, number }`
// This is a port of Google Android `libphonenumber`'s
// `phonenumberutil.js` of 17th November, 2016.
//
// https://github.com/googlei18n/libphonenumber/commits/master/javascript/i18n/phonenumbers/phonenumberutil.js

function parse(text, options) {
	if (typeof options === 'string') {
		var restrict_to_country = options;

		options = (0, _extends3.default)({}, default_options, {

			country: {
				restrict: restrict_to_country
			}
		});
	}

	if (!options) {
		options = (0, _extends3.default)({}, default_options);
	}

	// Parse the phone number

	if (!text || text.length > MAX_INPUT_STRING_LENGTH) {
		return {};
	}

	text = extract_possible_number(text);

	var _extract_country_phon = extract_country_phone_code(text);

	var country_phone_code = _extract_country_phon.country_phone_code;
	var number = _extract_country_phon.number;

	// Maybe invalid country phone code encountered

	if (!country_phone_code && !number) {
		return {};
	}

	var country = void 0;
	var country_metadata = void 0;

	if (country_phone_code) {
		// Check country restriction
		if (options.country.restrict && country_phone_code !== (0, _metadata3.get_phone_code)(_metadata2.default.countries[options.country.restrict])) {
			return {};
		}

		country_metadata = get_metadata_by_country_phone_code(country_phone_code);
	} else if (options.country.default || options.country.restrict) {
		country = options.country.default || options.country.restrict;
		country_metadata = _metadata2.default.countries[country];
		number = normalize(text);
	}

	if (!country_metadata) {
		return {};
	}

	// Sanity check
	if (number.length < MIN_LENGTH_FOR_NSN) {
		return {};
	}

	var national_number = strip_national_prefix(number, country_metadata);

	if (!country) {
		country = find_country_code(country_phone_code, national_number);

		// Check country restriction
		if (options.country.restrict && country !== options.country.restrict) {
			return {};
		}
	}

	// They say that sometimes national (significant) numbers
	// can be longer than `MAX_LENGTH_FOR_NSN` (e.g. in Germany).
	// https://github.com/googlei18n/libphonenumber/blob/7e1748645552da39c4e1ba731e47969d97bdb539/resources/phonenumber.proto#L36
	if (national_number.length < MIN_LENGTH_FOR_NSN || national_number.length > MAX_LENGTH_FOR_NSN) {
		return { phone: number };
	}

	var national_number_rule = new RegExp((0, _metadata3.get_national_number_pattern)(country_metadata));

	if (!(0, _common.matches_entirely)(national_number_rule, national_number)) {
		return {};
	}

	return { country: country, phone: national_number };
}

var PLUS_CHARS = '+＋';

// Digits accepted in phone numbers
// (ascii, fullwidth, arabic-indic, and eastern arabic digits).
var VALID_DIGITS = '0-9０-９٠-٩۰-۹';

// This consists of the plus symbol, digits, and arabic-indic digits.
var PHONE_NUMBER_START_PATTERN = new RegExp('[' + PLUS_CHARS + VALID_DIGITS + ']');

// Regular expression of trailing characters that we want to remove. We remove
// all characters that are not alpha or numerical characters. The hash character
// is retained here, as it may signify the previous block was an extension.
var AFTER_PHONE_NUMBER_END_PATTERN = new RegExp('[^' + VALID_DIGITS + ']+$');

var LEADING_PLUS_CHARS_PATTERN = new RegExp('^[' + PLUS_CHARS + ']+');

// These mappings map a character (key) to a specific digit that should
// replace it for normalization purposes. Non-European digits that
// may be used in phone numbers are mapped to a European equivalent.
var DIGIT_MAPPINGS = {
	'0': '0',
	'1': '1',
	'2': '2',
	'3': '3',
	'4': '4',
	'5': '5',
	'6': '6',
	'7': '7',
	'8': '8',
	'9': '9',
	'０': '0', // Fullwidth digit 0
	'１': '1', // Fullwidth digit 1
	'２': '2', // Fullwidth digit 2
	'３': '3', // Fullwidth digit 3
	'４': '4', // Fullwidth digit 4
	'５': '5', // Fullwidth digit 5
	'６': '6', // Fullwidth digit 6
	'７': '7', // Fullwidth digit 7
	'８': '8', // Fullwidth digit 8
	'９': '9', // Fullwidth digit 9
	'٠': '0', // Arabic-indic digit 0
	'١': '1', // Arabic-indic digit 1
	'٢': '2', // Arabic-indic digit 2
	'٣': '3', // Arabic-indic digit 3
	'٤': '4', // Arabic-indic digit 4
	'٥': '5', // Arabic-indic digit 5
	'٦': '6', // Arabic-indic digit 6
	'٧': '7', // Arabic-indic digit 7
	'٨': '8', // Arabic-indic digit 8
	'٩': '9', // Arabic-indic digit 9
	'۰': '0', // Eastern-Arabic digit 0
	'۱': '1', // Eastern-Arabic digit 1
	'۲': '2', // Eastern-Arabic digit 2
	'۳': '3', // Eastern-Arabic digit 3
	'۴': '4', // Eastern-Arabic digit 4
	'۵': '5', // Eastern-Arabic digit 5
	'۶': '6', // Eastern-Arabic digit 6
	'۷': '7', // Eastern-Arabic digit 7
	'۸': '8', // Eastern-Arabic digit 8
	'۹': '9' // Eastern-Arabic digit 9
};

// The maximum length of the country calling code.
var MAX_LENGTH_COUNTRY_CODE = 3;

// The minimum length of the national significant number.
var MIN_LENGTH_FOR_NSN = 2;

// The ITU says the maximum length should be 15,
// but one can find longer numbers in Germany.
var MAX_LENGTH_FOR_NSN = 17;

// We don't allow input strings for parsing to be longer than 250 chars.
// This prevents malicious input from consuming CPU.
var MAX_INPUT_STRING_LENGTH = 250;

// Normalizes a string of characters representing a phone number.
// This converts wide-ascii and arabic-indic numerals to European numerals,
// and strips punctuation and alpha characters.
function normalize(number) {
	return replace_characters(number, DIGIT_MAPPINGS);
}

// For any character not being part of `replacements`
// it is removed from the phone number.
function replace_characters(text, replacements) {
	var replaced = '';

	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = (0, _getIterator3.default)(text), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var character = _step.value;

			var replacement = replacements[character.toUpperCase()];

			if (replacement !== undefined) {
				replaced += replacement;
			}
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}

	return replaced;
}

// Attempts to extract a possible number from the string passed in.
function extract_possible_number(text) {
	var starts_at = text.search(PHONE_NUMBER_START_PATTERN);

	if (starts_at < 0) {
		return '';
	}

	return text
	// Trim everything to the left of the phone number
	.slice(starts_at)
	// Remove trailing non-numerical characters
	.replace(AFTER_PHONE_NUMBER_END_PATTERN, '');
}

// Tries to extract a country calling code from a number
function extract_country_phone_code(number) {
	if (!number) {
		return {};
	}

	// If this is not an international phone number,
	// then don't extract country phone code.
	if (!LEADING_PLUS_CHARS_PATTERN.test(number)) {
		return { number: number };
	}

	// Strip the leading '+' and remove non-digits
	number = normalize(number.replace(LEADING_PLUS_CHARS_PATTERN, ''));

	if (!number) {
		return {};
	}

	// Country codes do not begin with a '0'
	if (number[0] === '0') {
		return {};
	}

	// The thing with country phone codes
	// is that they are orthogonal to each other
	// i.e. there's no such country phone code A
	// for which country phone code B exists
	// where B starts with A.
	// Therefore, while scanning digits,
	// if a valid country code is found,
	// that means that it is the country code.
	//
	var i = 1;
	while (i <= MAX_LENGTH_COUNTRY_CODE && i <= number.length) {
		var country_phone_code = number.slice(0, i);

		if (_metadata2.default.country_phone_code_to_countries[country_phone_code]) {
			return { country_phone_code: country_phone_code, number: number.slice(i) };
		}

		i++;
	}

	return {};
}

// Formatting information for regions which share
// a country calling code is contained by only one region
// for performance reasons. For example, for NANPA region
// ("North American Numbering Plan Administration",
//  which includes USA, Canada, Cayman Islands, Bahamas, etc)
// it will be contained in the metadata for `US`.
function get_metadata_by_country_phone_code(country_phone_code) {
	var country_code = _metadata2.default.country_phone_code_to_countries[country_phone_code][0];
	return _metadata2.default.countries[country_code];
}

// Strips any national prefix (such as 0, 1) present in the number provided
function strip_national_prefix(number, country_metadata) {
	var national_prefix_for_parsing = (0, _metadata3.get_national_prefix_for_parsing)(country_metadata);

	if (!number || !national_prefix_for_parsing) {
		return number;
	}

	// Attempt to parse the first digits as a national prefix
	var national_prefix_pattern = new RegExp('^(?:' + national_prefix_for_parsing + ')');
	var national_prefix_matcher = national_prefix_pattern.exec(number);

	if (!national_prefix_matcher) {
		return number;
	}

	var national_prefix_transform_rule = (0, _metadata3.get_national_prefix_transform_rule)(country_metadata);

	var national_significant_number = void 0;

	// `!national_prefix_matcher[national_prefix_matcher.length - 1]`
	// implies nothing was captured by the capturing groups
	// in `national_prefix_for_parsing`.
	// Therefore, no transformation is necessary,
	// and we just remove the national prefix.
	if (!national_prefix_transform_rule || !national_prefix_matcher[national_prefix_matcher.length - 1]) {
		national_significant_number = number.slice(national_prefix_matcher[0].length);
	} else {
		national_significant_number = number.replace(national_prefix_pattern, national_prefix_transform_rule);
	}

	var national_number_rule = new RegExp((0, _metadata3.get_national_number_pattern)(country_metadata));

	// If the original number was viable, and the resultant number is not, then return.
	if ((0, _common.matches_entirely)(national_number_rule, number) && !(0, _common.matches_entirely)(national_number_rule, national_significant_number)) {
		return number;
	}

	return national_significant_number;
}

function find_country_code(country_phone_code, national_phone_number) {
	if (!country_phone_code) {
		return;
	}

	var possible_country_codes = _metadata2.default.country_phone_code_to_countries[country_phone_code];

	if (!possible_country_codes) {
		return;
	}

	var _iteratorNormalCompletion2 = true;
	var _didIteratorError2 = false;
	var _iteratorError2 = undefined;

	try {
		for (var _iterator2 = (0, _getIterator3.default)(possible_country_codes), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
			var country_code = _step2.value;

			var country = _metadata2.default.countries[country_code];

			if ((0, _metadata3.get_leading_digits)(country)) {
				if (national_phone_number && national_phone_number.search((0, _metadata3.get_leading_digits)(country)) === 0) {
					return country_code;
				}
			} else if (is_national_phone_number(national_phone_number, country)) {
				return country_code;
			}
		}
	} catch (err) {
		_didIteratorError2 = true;
		_iteratorError2 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion2 && _iterator2.return) {
				_iterator2.return();
			}
		} finally {
			if (_didIteratorError2) {
				throw _iteratorError2;
			}
		}
	}
}

function is_national_phone_number(national_number, country_metadata) {
	if (!national_number) {
		return false;
	}

	// Faster false positives
	// const possible_lengths = [...]
	// if (possible_lengths && possible_lengths.length > 0 &&
	// 	possible_lengths.indexOf(national_number.length) < 0)
	// {
	// 	return false
	// }

	return (0, _common.matches_entirely)((0, _metadata3.get_national_number_pattern)(country_metadata), national_number);
}
//# sourceMappingURL=parse.js.map